﻿//////////////////////////////////////////////////////////////////////////////////
//	MultipleWiimoteForm.cs
//	Managed Wiimote Library Tester
//	Written by Brian Peek (http://www.brianpeek.com/)
//  for MSDN's Coding4Fun (http://msdn.microsoft.com/coding4fun/)
//	Visit http://blogs.msdn.com/coding4fun/archive/2007/03/14/1879033.aspx
//  and http://www.codeplex.com/WiimoteLib
//  for more information
//////////////////////////////////////////////////////////////////////////////////

using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using WiimoteLib;

namespace WiimoteTest
{
	public partial class WiimoteInfo : UserControl
	{
		private delegate void UpdateWiimoteStateDelegate(WiimoteChangedEventArgs args);
		private delegate void UpdateExtensionChangedDelegate(WiimoteExtensionChangedEventArgs args);

		private Bitmap b = new Bitmap(512, 384, PixelFormat.Format24bppRgb);
		private Graphics g;
		private Wiimote mWiimote;

        private System.IO.FileInfo outFile;
        private System.IO.StreamWriter writer;
        private DateTime startTime;
        private bool isRecording;
        private long lastTime;
        private int fileNameInc;
        
        public void startRecording()
        {
            //String fName = "c:/temp/wii" + DateTime.Now.Day + DateTime.Now.Month + DateTime.Now.Year + this.fileNameInc+".txt"; 
            String fName = this.outPutFile.Text;
            outFile = new System.IO.FileInfo(fName);
            this.fileNameInc += 1;
            writer = outFile.CreateText();
            //startTime = DateTime.Now;
            startTime = MultipleWiimoteForm.getStartTime();
            isRecording = true;
            lastTime = 0;
            this.labelRecording.Text = "RECORDING";
        }



        private void record(IRState s)
        {
            if (!isRecording)
                return;

            TimeSpan timeSpan = DateTime.Now - startTime;
            long t = timeSpan.Milliseconds + timeSpan.Seconds * 1000 + timeSpan.Minutes * 1000 * 60 + timeSpan.Hours * 1000 * 60 * 60;

            if (t - lastTime < 2)
                return;  /// make sure my data is at least 2 millisecond apart

            lastTime = t;

            writer.WriteLine(t + "\t" + s.IRSensors[0].RawPosition.X + "\t" + s.IRSensors[0].RawPosition.Y
                               + "\t" + s.IRSensors[1].RawPosition.X + "\t" + s.IRSensors[1].RawPosition.Y
                               + "\t" + s.IRSensors[2].RawPosition.X + "\t" + s.IRSensors[2].RawPosition.Y
                               + "\t" + s.IRSensors[3].RawPosition.X + "\t" + s.IRSensors[3].RawPosition.Y);
        }

        public void stopRecording()
        {
            writer.Close();
            isRecording = false;
            this.labelRecording.Text = "Not Recording";
            String fName = this.outPutFile.Text;
            fName = fName.Remove(fName.Length - 5);
            fName += this.fileNameInc + ".txt";
            this.outPutFile.Text = fName;
        }

        public void setFname(String fName)
        {
            outPutFile.Text = fName;
        }

		public WiimoteInfo()
		{
			InitializeComponent();
			g = Graphics.FromImage(b);
            isRecording = false;
            fileNameInc = 0;

        }

		public WiimoteInfo(Wiimote wm) : this()
		{
			mWiimote = wm;
		}

		public void UpdateState(WiimoteChangedEventArgs args)
		{
            try
            {

                BeginInvoke(new UpdateWiimoteStateDelegate(UpdateWiimoteChanged), args);
            }
            catch (Exception) { }
        }

		public void UpdateExtension(WiimoteExtensionChangedEventArgs args)
		{
			BeginInvoke(new UpdateExtensionChangedDelegate(UpdateExtensionChanged), args);
		}


		private void UpdateWiimoteChanged(WiimoteChangedEventArgs args)
		{
			WiimoteState ws = args.WiimoteState;


			g.Clear(Color.Black);
            record(ws.IRState);
            
			UpdateIR(ws.IRState.IRSensors[0], lblIR1, lblIR1Raw, chkFound1, Color.Red);
			UpdateIR(ws.IRState.IRSensors[1], lblIR2, lblIR2Raw, chkFound2, Color.White);
			UpdateIR(ws.IRState.IRSensors[2], lblIR3, lblIR3Raw, chkFound3, Color.Yellow);
			UpdateIR(ws.IRState.IRSensors[3], lblIR4, lblIR4Raw, chkFound4, Color.DeepSkyBlue);

            debugText.Text = ws.IRState.Mode.ToString()  
                +  "\n" + ws.IRState.IRSensors[0]
                + "\n" + ws.IRState.IRSensors[1]
                + "\n" + ws.IRState.IRSensors[2]
                + "\n" + ws.IRState.IRSensors[3];


            WiimoteLib.Point ir0 = ws.IRState.IRSensors[0].RawPosition;
            WiimoteLib.Point ir1 = ws.IRState.IRSensors[1].RawPosition;
            WiimoteLib.Point ir2 = ws.IRState.IRSensors[2].RawPosition;
            WiimoteLib.Point ir3 = ws.IRState.IRSensors[3].RawPosition;

			if(ws.IRState.IRSensors[0].Found && ws.IRState.IRSensors[1].Found)
                g.DrawLine(new Pen(Color.Green), (int)(ir0.X / 2), (int)(ir0.Y / 2)
                    , (int)(ir1.X / 2), (int)(ir1.Y / 2));

            if (ws.IRState.IRSensors[1].Found && ws.IRState.IRSensors[2].Found)
                g.DrawLine(new Pen(Color.Green,2), (int)(ir1.X / 2), (int)(ir1.Y / 2)
                    , (int)(ir2.X / 2), (int)(ir2.Y / 2));

            if (ws.IRState.IRSensors[2].Found && ws.IRState.IRSensors[3].Found)
                g.DrawLine(new Pen(Color.Green), (int)(ir2.X / 2), (int)(ir2.Y / 2)
                    , (int)(ir3.X / 2), (int)(ir3.Y / 2));

			pbIR.Image = b;

			pbBattery.Value = (ws.Battery > 0xc8 ? 0xc8 : (int)ws.Battery);
			lblBattery.Text = ws.Battery.ToString();
			lblDevicePath.Text = "Device Path: " + mWiimote.HIDDevicePath;
		}

		private void UpdateIR(IRSensor irSensor, Label lblNorm, Label lblRaw, CheckBox chkFound, Color color)
		{
			chkFound.Checked = irSensor.Found;

			if(irSensor.Found)
			{
				lblNorm.Text = irSensor.Position.ToString() + ", " + irSensor.Size;
				lblRaw.Text = irSensor.RawPosition.ToString();
                g.DrawEllipse(new Pen(color, 4), (int)(irSensor.RawPosition.X / 2), (int)(irSensor.RawPosition.Y / 2),
							 irSensor.Size+8, irSensor.Size+8);

			}
		}

		private void UpdateExtensionChanged(WiimoteExtensionChangedEventArgs args)
		{
		}

		public Wiimote Wiimote
		{
			set { mWiimote = value; }
		}

        private void chkExtension_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void pbIR_Click(object sender, EventArgs e)
        {

        }

        private void chkC_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void debugText_Click(object sender, EventArgs e)
        {

        }

        private void groupBox4_Enter(object sender, EventArgs e)
        {

        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            this.startRecording();
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            this.stopRecording();
        }
	}
}
